import argparse

from .. import version


def make_parser():
    parser = argparse.ArgumentParser(
        description="Read in a file or set of files, and return the result.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument("path", nargs="+", help="Path of a file or a folder of files.")
    parser.add_argument(
        "-r",
        "--rename",
        help="Rename comic archive from the files metadata.",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "-o",
        "--online",
        help="Search online and attempt to identify comic archive.",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--id", help="Identify file for tagging with the Metron Issue Id."
    )
    parser.add_argument(
        "-d",
        "--delete",
        help="Delete the metadata tags from the file.",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--ignore-existing",
        help="Ignore files that have existing metadata tag.",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--missing",
        help="List files without metadata.",
        action="store_true",
        default=False,
    )
    parser.add_argument("-u", "--user", help="Metron user identity")
    parser.add_argument("-p", "--password", help="Metron user password")
    parser.add_argument(
        "--set-metron-user", help="Save the Metron user settings", action="store_true"
    )
    parser.add_argument(
        "-s",
        "--sort",
        help="Sort files that contain metadata tags.",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--sort-dir",
        help="Directory wher files should be sorted to. This should not have a trailing path serparator.",
    )
    parser.add_argument(
        "--set-sort-dir",
        help="Set the directory where files should be sorted.",
        action="store_true",
    )
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s {}".format(version),
        help="Show the version number and exit",
    )

    return parser



# coding=utf-8
"""Some generic utilities"""

# Copyright 2012-2014 Anthony Beville

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os


def get_recursive_filelist(pathlist):
    """Get a recursive list of of all files under all path items in the list"""
    filelist = []
    for p in pathlist:
        # if path is a folder, walk it recursively, and all files underneath
        if isinstance(p, str):
            pass
        elif not isinstance(p, str):
            # it's probably a QString
            p = str(p)

        if os.path.isdir(p):
            for root, _, files in os.walk(p):
                for f in files:
                    if isinstance(f, str):
                        pass
                    elif not isinstance(f, str):
                        # it's probably a QString
                        f = str(f)
                    filelist.append(os.path.join(root, f))
        else:
            filelist.append(p)

        # Sort the list before returning it
        filelist.sort()

    return filelist


def listToString(l):
    string = ""
    if l is not None:
        for item in l:
            if len(string) > 0:
                string += "; "
            string += item
    return string


def removearticles(text):
    text = text.lower()
    articles = ["and", "a", "&", "issue", "the"]
    newText = ""
    for word in text.split(" "):
        if word not in articles:
            newText += word + " "

    newText = newText[:-1]

    # now get rid of some other junk
    newText = newText.replace(":", "")
    newText = newText.replace(",", "")
    newText = newText.replace("-", " ")

    # since the CV API changed, searches for series names with periods
    # now explicitly require the period to be in the search key,
    # so the line below is removed (for now)
    # newText = newText.replace(".", "")

    return newText


def unique_file(file_name):
    counter = 1
    # returns ('/path/file', '.ext')
    file_name_parts = os.path.splitext(file_name)
    while True:
        if not os.path.lexists(file_name):
            return file_name
        file_name = file_name_parts[0] + " (" + str(counter) + ")" + file_name_parts[1]
        counter += 1



import tempfile
from unittest import TestCase, main

from metrontagger.taggerlib.utils import cleanup_string, create_issue_query_dict


class TestTaggerlibUtils(TestCase):
    def setUp(self):
        self.series = "Aquaman"
        self.number = "1"
        self.year = "1962"
        self.tmp_file = tempfile.NamedTemporaryFile(
            prefix=f"{self.series} #{self.number} ({self.year})", suffix=".cbz"
        )

    def test_cleanup_colon_space(self):
        test_str = "Hashtag: Danger (2019)"
        res = cleanup_string(test_str)
        self.assertEqual(res, "Hashtag - Danger (2019)")

    def test_cleanup_space_colon(self):
        test_str = "Hashtag :Danger (2019)"
        res = cleanup_string(test_str)
        self.assertEqual(res, "Hashtag -Danger (2019)")

    def test_cleanup_colon(self):
        test_str = "Hashtag:Danger (2019)"
        res = cleanup_string(test_str)
        self.assertEqual(res, "Hashtag-Danger (2019)")

    def test_cleanup_backslash(self):
        test_str = "Hack/Slash (2019)"
        res = cleanup_string(test_str)
        self.assertEqual(res, "Hack-Slash (2019)")

    def test_cleanup_questionmark(self):
        test_str = "What If? (2019)"
        res = cleanup_string(test_str)
        self.assertEqual(res, "What If (2019)")

    def test_create_issue_query_dict(self):
        i = create_issue_query_dict(self.tmp_file.name)
        results = {
            "series": f"{self.series}",
            "volume": "",
            "number": f"{self.number}",
            "year": f"{self.year}",
        }
        self.assertEqual(i, results)


if __name__ == "__main__":
    main()



import json
import platform
import time
from datetime import datetime
from urllib.error import HTTPError
from urllib.request import Request, urlopen

from ratelimit import limits, sleep_and_retry

from metrontagger.comicapi.genericmetadata import GenericMetadata
from metrontagger.comicapi.issuestring import IssueString
from metrontagger.comicapi.utils import listToString

from .. import version

ONE_MINUTE = 60


class MetronTalker:
    def __init__(self, auth):
        self.api_base_url = "https://metron.cloud/api"
        self.auth_str = f"Basic {auth.decode('utf-8')}"
        self.user_agent = (
            f"Metron-Tagger/{version} ({platform.system()}; {platform.release()})"
        )

    @classmethod
    def parseDateStr(self, date_str):
        day = None
        month = None
        year = None
        if date_str is not None:
            parts = date_str.split("-")
            year = parts[0]
            if len(parts) > 1:
                month = parts[1]
                if len(parts) > 2:
                    day = parts[2]
        return day, month, year

    @sleep_and_retry
    @limits(calls=20, period=ONE_MINUTE)
    def fetchResponse(self, url):
        if url.lower().startswith("https"):
            request = Request(url)
        else:
            raise ValueError from None

        request.add_header("Authorization", self.auth_str)
        request.add_header("User-Agent", self.user_agent)

        try:
            content = urlopen(request)
        except HTTPError as e:
            # TODO: Look into handling throttling better, but for now let's use this.
            if e.code == 429:
                print("Exceeded api rate limit. Sleeping for 30 seconds...")
                time.sleep(30)
                return self.fetchResponse(url)
            raise

        resp = json.loads(content.read().decode("utf-8"))

        return resp

    def fetchIssueDataByIssueId(self, issue_id):
        url = self.api_base_url + f"/issue/{issue_id}/?format=json"
        resp = self.fetchResponse(url)
        md = self.mapMetronDataToMetadata(resp)
        md.isEmpty = False

        return md

    def searchForIssue(self, query_dict):
        url = (
            self.api_base_url
            + f"/issue/?series_name={query_dict['series']}&number={query_dict['number']}"
        )
        if query_dict["volume"]:
            url += f"&series_volume={query_dict['volume']}"
        if query_dict["year"]:
            url += f"&cover_year={query_dict['year']}"
        url += "&format=json"
        resp = self.fetchResponse(url)

        return resp

    def mapMetronDataToMetadata(self, issue_results):
        metadata = GenericMetadata()

        metadata.series = issue_results["series"]["name"]
        metadata.volume = issue_results["volume"]

        num_s = IssueString(issue_results["number"]).asString()

        metadata.issue = num_s

        titles = issue_results["name"]
        title_list = []
        for title in titles:
            title_list.append(title)
        metadata.title = listToString(title_list)

        metadata.publisher = issue_results["publisher"]["name"]
        metadata.day, metadata.month, metadata.year = self.parseDateStr(
            issue_results["cover_date"]
        )

        metadata.comments = issue_results["desc"]

        d = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        metadata.notes = (
            f"Tagged with MetronTagger-{version} using info from Metron on {d}. "
            + f"[issue_id:{issue_results['id']}]"
        )

        person_credits = issue_results["credits"]
        for person in person_credits:
            if "role" in person:
                roles = person["role"]
                for role in roles:
                    metadata.addCredit(person["creator"], role["name"], False)

        character_credits = issue_results["characters"]
        character_list = []
        for character in character_credits:
            character_list.append(character["name"])
        metadata.characters = listToString(character_list)

        team_credits = issue_results["teams"]
        team_list = []
        for team in team_credits:
            team_list.append(team["name"])
        metadata.teams = listToString(team_list)

        story_arc_credits = issue_results["arcs"]
        arc_list = []
        for arc in story_arc_credits:
            arc_list.append(arc["name"])
        if len(arc_list) > 0:
            metadata.storyArc = listToString(arc_list)

        return metadata



"""Functions for parsing comic info from filename

This should probably be re-written, but, well, it mostly works!
"""

# Copyright 2012-2014 Anthony Beville

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Some portions of this code were modified from pyComicMetaThis project
# http://code.google.com/p/pycomicmetathis/

import re
import os
from urllib.parse import unquote


class FileNameParser:
    @staticmethod
    def repl(m):
        return " " * len(m.group())

    def fixSpaces(self, string, remove_dashes=True):
        if remove_dashes:
            placeholders = ["[-_]", "  +"]
        else:
            placeholders = ["[_]", "  +"]
        for ph in placeholders:
            string = re.sub(ph, self.repl, string)
        return string  # .strip()

    def getIssueCount(self, filename, issue_end):

        count = ""
        filename = filename[issue_end:]

        # replace any name separators with spaces
        tmpstr = self.fixSpaces(filename)
        found = False

        match = re.search(r"(?<=\sof\s)\d+(?=\s)", tmpstr, re.IGNORECASE)
        if match:
            count = match.group()
            found = True

        if not found:
            match = re.search(r"(?<=\(of\s)\d+(?=\))", tmpstr, re.IGNORECASE)
            if match:
                count = match.group()
                found = True

        count = count.lstrip("0")

        return count

    def getIssueNumber(self, filename):
        """Returns a tuple of issue number string, and start and end indexes in the filename
        (The indexes will be used to split the string up for further parsing)
        """

        found = False
        issue = ""
        start = 0
        end = 0

        # first, look for multiple "--", this means it's formatted differently
        # from most:
        if "--" in filename:
            # the pattern seems to be that anything to left of the first "--"
            # is the series name followed by issue
            filename = re.sub("--.*", self.repl, filename)

        elif "__" in filename:
            # the pattern seems to be that anything to left of the first "__"
            # is the series name followed by issue
            filename = re.sub("__.*", self.repl, filename)

        filename = filename.replace("+", " ")

        # replace parenthetical phrases with spaces
        filename = re.sub(r"\(.*?\)", self.repl, filename)
        filename = re.sub(r"\[.*?\]", self.repl, filename)

        # replace any name separators with spaces
        filename = self.fixSpaces(filename)

        # remove any "of NN" phrase with spaces (problem: this could break on
        # some titles)
        filename = re.sub(r"of [\d]+", self.repl, filename)

        # print u"[{0}]".format(filename)

        # we should now have a cleaned up filename version with all the words in
        # the same positions as original filename

        # make a list of each word and its position
        word_list = list()
        for m in re.finditer(r"\S+", filename):
            word_list.append((m.group(0), m.start(), m.end()))

        # remove the first word, since it can't be the issue number
        if len(word_list) > 1:
            word_list = word_list[1:]
        else:
            # only one word??  just bail.
            return issue, start, end

        # Now try to search for the likely issue number word in the list

        # first look for a word with "#" followed by digits with optional suffix
        # this is almost certainly the issue number
        for w in reversed(word_list):
            if re.match(r"#[-]?(([0-9]*\.[0-9]+|[0-9]+)(\w*))", w[0]):
                found = True
                break

        # same as above but w/o a '#', and only look at the last word in the
        # list
        if not found:
            w = word_list[-1]
            if re.match(r"[-]?(([0-9]*\.[0-9]+|[0-9]+)(\w*))", w[0]):
                found = True

        # now try to look for a # followed by any characters
        if not found:
            for w in reversed(word_list):
                if re.match(r"#\S+", w[0]):
                    found = True
                    break

        if found:
            issue = w[0]
            start = w[1]
            end = w[2]
            if issue[0] == "#":
                issue = issue[1:]

        return issue, start, end

    def getSeriesName(self, filename, issue_start):
        """Use the issue number string index to split the filename string"""

        if issue_start != 0:
            filename = filename[:issue_start]

        # in case there is no issue number, remove some obvious stuff
        if "--" in filename:
            # the pattern seems to be that anything to left of the first "--"
            # is the series name followed by issue
            filename = re.sub("--.*", self.repl, filename)

        elif "__" in filename:
            # the pattern seems to be that anything to left of the first "__"
            # is the series name followed by issue
            filename = re.sub("__.*", self.repl, filename)

        filename = filename.replace("+", " ")
        tmpstr = self.fixSpaces(filename, remove_dashes=False)

        series = tmpstr
        volume = ""

        # save the last word
        try:
            last_word = series.split()[-1]
        except ValueError:
            last_word = ""

        # remove any parenthetical phrases
        series = re.sub(r"\(.*?\)", "", series)

        # search for volume number
        match = re.search(r"(.+)([vV]|[Vv][oO][Ll]\.?\s?)(\d+)\s*$", series)
        if match:
            series = match.group(1)
            volume = match.group(3)

        # if a volume wasn't found, see if the last word is a year in parentheses
        # since that's a common way to designate the volume
        if volume == "":
            # match either (YEAR), (YEAR-), or (YEAR-YEAR2)
            match = re.search(r"(\()(\d{4})(-(\d{4}|)|)(\))", last_word)
            if match:
                volume = match.group(2)

        series = series.strip()

        # if we don't have an issue number (issue_start==0), look
        # for hints i.e. "TPB", "one-shot", "OS", "OGN", etc that might
        # be removed to help search online
        if issue_start == 0:
            one_shot_words = ["tpb", "os", "one-shot", "ogn", "gn"]
            try:
                last_word = series.split()[-1]
                if last_word.lower() in one_shot_words:
                    series = series.rsplit(" ", 1)[0]
            except ValueError:
                pass

        return series, volume.strip()

    @staticmethod
    def getYear(filename, issue_end):

        filename = filename[issue_end:]

        year = ""
        # look for four digit number with "(" ")" or "--" around it
        match = re.search(r"(\(\d\d\d\d\))|(--\d\d\d\d--)", filename)
        if match:
            year = match.group()
            # remove non-digits
            year = re.sub("[^0-9]", "", year)
        return year

    def getRemainder(self, filename, year, count, volume, issue_end):
        """Make a guess at where the the non-interesting stuff begins"""

        remainder = ""

        if "--" in filename:
            remainder = filename.split("--", 1)[1]
        elif "__" in filename:
            remainder = filename.split("__", 1)[1]
        elif issue_end != 0:
            remainder = filename[issue_end:]

        remainder = self.fixSpaces(remainder, remove_dashes=False)
        if volume != "":
            remainder = remainder.replace("Vol." + volume, "", 1)
        if year != "":
            remainder = remainder.replace(year, "", 1)
        if count != "":
            remainder = remainder.replace("of " + count, "", 1)

        remainder = remainder.replace("()", "")
        remainder = remainder.replace("  ", " ")  # cleans some whitespace mess

        return remainder.strip()

    def parseFilename(self, filename):

        # remove the path
        filename = os.path.basename(filename)

        # remove the extension
        filename = os.path.splitext(filename)[0]

        # url decode, just in case
        filename = unquote(filename)

        # sometimes archives get messed up names from too many decodes
        # often url encodings will break and leave "_28" and "_29" in place
        # of "(" and ")"  see if there are a number of these, and replace them
        if filename.count("_28") > 1 and filename.count("_29") > 1:
            filename = filename.replace("_28", "(")
            filename = filename.replace("_29", ")")

        self.issue, issue_start, issue_end = self.getIssueNumber(filename)
        self.series, self.volume = self.getSeriesName(filename, issue_start)

        # provides proper value when the filename doesn't have a issue number
        if issue_end == 0:
            issue_end = len(self.series)

        self.year = self.getYear(filename, issue_end)
        self.issue_count = self.getIssueCount(filename, issue_end)
        self.remainder = self.getRemainder(
            filename, self.year, self.issue_count, self.volume, issue_end
        )

        if self.issue != "":
            # strip off leading zeros
            self.issue = self.issue.lstrip("0")
            if self.issue == "":
                self.issue = "0"
            if self.issue[0] == ".":
                self.issue = "0" + self.issue



"""A class to encapsulate ComicRack's ComicInfo.xml data"""

# Copyright 2012-2014 Anthony Beville

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import xml.etree.ElementTree as ET

from .genericmetadata import GenericMetadata
from . import utils


class ComicInfoXml:

    writer_synonyms = ["writer", "plotter", "scripter"]
    penciller_synonyms = ["artist", "penciller", "penciler", "breakdowns"]
    inker_synonyms = ["inker", "artist", "finishes"]
    colorist_synonyms = ["colorist", "colourist", "colorer", "colourer"]
    letterer_synonyms = ["letterer"]
    cover_synonyms = ["cover", "covers", "coverartist", "cover artist"]
    editor_synonyms = ["editor"]

    def getParseableCredits(self):
        parsable_credits = []
        parsable_credits.extend(self.writer_synonyms)
        parsable_credits.extend(self.penciller_synonyms)
        parsable_credits.extend(self.inker_synonyms)
        parsable_credits.extend(self.colorist_synonyms)
        parsable_credits.extend(self.letterer_synonyms)
        parsable_credits.extend(self.cover_synonyms)
        parsable_credits.extend(self.editor_synonyms)
        return parsable_credits

    def metadataFromString(self, string):

        tree = ET.ElementTree(ET.fromstring(string))
        return self.convertXMLToMetadata(tree)

    def stringFromMetadata(self, metadata):

        header = '<?xml version="1.0"?>\n'

        tree = self.convertMetadataToXML(self, metadata)
        tree_str = ET.tostring(tree.getroot()).decode()
        return header + tree_str

    def indent(self, elem, level=0):
        # for making the XML output readable
        i = "\n" + level * "  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.indent(elem, level + 1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    def convertMetadataToXML(self, filename, metadata):

        # shorthand for the metadata
        md = metadata

        # build a tree structure
        root = ET.Element("ComicInfo")
        root.attrib["xmlns:xsi"] = "http://www.w3.org/2001/XMLSchema-instance"
        root.attrib["xmlns:xsd"] = "http://www.w3.org/2001/XMLSchema"
        # helper func

        def assign(cix_entry, md_entry):
            if md_entry is not None:
                ET.SubElement(root, cix_entry).text = "{0}".format(md_entry)

        assign("Title", md.title)
        assign("Series", md.series)
        assign("Number", md.issue)
        assign("Count", md.issueCount)
        assign("Volume", md.volume)
        assign("AlternateSeries", md.alternateSeries)
        assign("AlternateNumber", md.alternateNumber)
        assign("StoryArc", md.storyArc)
        assign("SeriesGroup", md.seriesGroup)
        assign("AlternateCount", md.alternateCount)
        assign("Summary", md.comments)
        assign("Notes", md.notes)
        assign("Year", md.year)
        assign("Month", md.month)
        assign("Day", md.day)

        # need to specially process the credits, since they are structured
        # differently than CIX
        credit_writer_list = list()
        credit_penciller_list = list()
        credit_inker_list = list()
        credit_colorist_list = list()
        credit_letterer_list = list()
        credit_cover_list = list()
        credit_editor_list = list()

        # first, loop thru credits, and build a list for each role that CIX
        # supports
        for credit in metadata.credits:

            if credit["role"].lower() in set(self.writer_synonyms):
                credit_writer_list.append(credit["person"].replace(",", ""))

            if credit["role"].lower() in set(self.penciller_synonyms):
                credit_penciller_list.append(credit["person"].replace(",", ""))

            if credit["role"].lower() in set(self.inker_synonyms):
                credit_inker_list.append(credit["person"].replace(",", ""))

            if credit["role"].lower() in set(self.colorist_synonyms):
                credit_colorist_list.append(credit["person"].replace(",", ""))

            if credit["role"].lower() in set(self.letterer_synonyms):
                credit_letterer_list.append(credit["person"].replace(",", ""))

            if credit["role"].lower() in set(self.cover_synonyms):
                credit_cover_list.append(credit["person"].replace(",", ""))

            if credit["role"].lower() in set(self.editor_synonyms):
                credit_editor_list.append(credit["person"].replace(",", ""))

        # second, convert each list to string, and add to XML struct
        if len(credit_writer_list) > 0:
            node = ET.SubElement(root, "Writer")
            node.text = utils.listToString(credit_writer_list)

        if len(credit_penciller_list) > 0:
            node = ET.SubElement(root, "Penciller")
            node.text = utils.listToString(credit_penciller_list)

        if len(credit_inker_list) > 0:
            node = ET.SubElement(root, "Inker")
            node.text = utils.listToString(credit_inker_list)

        if len(credit_colorist_list) > 0:
            node = ET.SubElement(root, "Colorist")
            node.text = utils.listToString(credit_colorist_list)

        if len(credit_letterer_list) > 0:
            node = ET.SubElement(root, "Letterer")
            node.text = utils.listToString(credit_letterer_list)

        if len(credit_cover_list) > 0:
            node = ET.SubElement(root, "CoverArtist")
            node.text = utils.listToString(credit_cover_list)

        if len(credit_editor_list) > 0:
            node = ET.SubElement(root, "Editor")
            node.text = utils.listToString(credit_editor_list)

        assign("Publisher", md.publisher)
        assign("Imprint", md.imprint)
        assign("Genre", md.genre)
        assign("Web", md.webLink)
        assign("PageCount", md.pageCount)
        assign("LanguageISO", md.language)
        assign("Format", md.format)
        assign("AgeRating", md.maturityRating)
        if md.blackAndWhite is not None and md.blackAndWhite:
            ET.SubElement(root, "BlackAndWhite").text = "Yes"
        assign("Manga", md.manga)
        assign("Characters", md.characters)
        assign("Teams", md.teams)
        assign("Locations", md.locations)
        assign("ScanInformation", md.scanInfo)

        #  loop and add the page entries under pages node
        if len(md.pages) > 0:
            pages_node = ET.SubElement(root, "Pages")
            for page_dict in md.pages:
                page_node = ET.SubElement(pages_node, "Page")
                page_node.attrib = page_dict

        # self pretty-print
        self.indent(root)

        # wrap it in an ElementTree instance, and save as XML
        tree = ET.ElementTree(root)
        return tree

    @classmethod
    def convertXMLToMetadata(cls, tree):

        root = tree.getroot()

        if root.tag != "ComicInfo":
            raise ValueError("Metadata is not ComicInfo format")

        metadata = GenericMetadata()
        md = metadata

        # Helper function
        def xlate(tag):
            node = root.find(tag)
            if node is not None:
                return node.text
            else:
                return None

        md.series = xlate("Series")
        md.title = xlate("Title")
        md.issue = xlate("Number")
        md.issueCount = xlate("Count")
        md.volume = xlate("Volume")
        md.alternateSeries = xlate("AlternateSeries")
        md.alternateNumber = xlate("AlternateNumber")
        md.alternateCount = xlate("AlternateCount")
        md.comments = xlate("Summary")
        md.notes = xlate("Notes")
        md.year = xlate("Year")
        md.month = xlate("Month")
        md.day = xlate("Day")
        md.publisher = xlate("Publisher")
        md.imprint = xlate("Imprint")
        md.genre = xlate("Genre")
        md.webLink = xlate("Web")
        md.language = xlate("LanguageISO")
        md.format = xlate("Format")
        md.manga = xlate("Manga")
        md.characters = xlate("Characters")
        md.teams = xlate("Teams")
        md.locations = xlate("Locations")
        md.pageCount = xlate("PageCount")
        md.scanInfo = xlate("ScanInformation")
        md.storyArc = xlate("StoryArc")
        md.seriesGroup = xlate("SeriesGroup")
        md.maturityRating = xlate("AgeRating")

        tmp = xlate("BlackAndWhite")
        md.blackAndWhite = False
        if tmp is not None and tmp.lower() in ["yes", "true", "1"]:
            md.blackAndWhite = True
        # Now extract the credit info
        for n in root:
            if (
                n.tag == "Writer"
                or n.tag == "Penciller"
                or n.tag == "Inker"
                or n.tag == "Colorist"
                or n.tag == "Letterer"
                or n.tag == "Editor"
            ):
                if n.text is not None:
                    for name in n.text.split(","):
                        metadata.addCredit(name.strip(), n.tag)

            if n.tag == "CoverArtist":
                if n.text is not None:
                    for name in n.text.split(","):
                        metadata.addCredit(name.strip(), "Cover")

        # parse page data now
        pages_node = root.find("Pages")
        if pages_node is not None:
            for page in pages_node:
                metadata.pages.append(page.attrib)
                # print page.attrib

        metadata.isEmpty = False

        return metadata

    def writeToExternalFile(self, filename, metadata):

        tree = self.convertMetadataToXML(self, metadata)
        # ET.dump(tree)
        tree.write(filename, encoding="utf-8")

    def readFromExternalFile(self, filename):

        tree = ET.parse(filename)
        return self.convertXMLToMetadata(tree)



import os
import tempfile
from shutil import make_archive
from unittest import TestCase, main

from metrontagger.comicapi.genericmetadata import GenericMetadata
from metrontagger.taggerlib.filerenamer import FileRenamer


class TestFileRenamer(TestCase):
    def setUp(self):
        self.tmp_archive_dir = tempfile.TemporaryDirectory()
        self.tmp_image_dir = tempfile.TemporaryDirectory()
        # Create 3 fake jpgs
        img_1 = tempfile.NamedTemporaryFile(
            suffix=".jpg", dir=self.tmp_image_dir.name, mode="wb"
        )
        img_1.write(b"test data")
        img_2 = tempfile.NamedTemporaryFile(
            suffix=".jpg", dir=self.tmp_image_dir.name, mode="wb"
        )
        img_2.write(b"more data")
        img_3 = tempfile.NamedTemporaryFile(
            suffix=".jpg", dir=self.tmp_image_dir.name, mode="wb"
        )
        img_3.write(b"yet more data")

        self.zipfile = os.path.join(self.tmp_archive_dir.name, "comic")

        # Create zipfile
        open(make_archive(self.zipfile, "zip", self.tmp_image_dir.name), "rb").read()

        # Create test metadata
        self.md = GenericMetadata()
        self.md.series = "Aquaman"
        self.md.issue = "1"
        self.md.year = "2011"

    def tearDown(self):
        self.tmp_archive_dir.cleanup()
        self.tmp_image_dir.cleanup()

    def test_determine_name(self):
        renamer = FileRenamer(self.md)
        new_name = renamer.determineName(self.zipfile + ".zip")
        self.assertEqual(new_name, "Aquaman #001 (2011).zip")


if __name__ == "__main__":
    main()



import configparser
import os
import platform


class MetronTaggerSettings:
    @staticmethod
    def getSettingsFolder():
        if platform.system() == "Windows":
            folder = os.path.join(os.environ["APPDATA"], "MetronTagger")
        else:
            folder = os.path.join(os.path.expanduser("~"), ".MetronTagger")
        if folder is not None:
            folder = folder
        return folder

    def setDefaultValues(self):
        # Metron creditials
        self.metron_user = ""
        self.metron_pass = ""
        self.sort_dir = ""

    def __init__(self, config_dir=None):
        self.settings_file = ""
        self.folder = ""
        self.setDefaultValues()

        self.config = configparser.ConfigParser()
        self.folder = config_dir or MetronTaggerSettings.getSettingsFolder()

        if not os.path.exists(self.folder):
            os.makedirs(self.folder)

        self.settings_file = os.path.join(self.folder, "settings.ini")

        # Write the config file if it doesn't exist
        if not os.path.exists(self.settings_file):
            self.save()
        else:
            self.load()

    def load(self):
        self.config.read(self.settings_file)

        if self.config.has_option("metron", "user"):
            self.metron_user = self.config["metron"]["user"]

        if self.config.has_option("metron", "password"):
            self.metron_pass = self.config["metron"]["password"]

        if self.config.has_option("DEFAULT", "sort_dir"):
            self.sort_dir = self.config["DEFAULT"]["sort_dir"]

    def save(self):
        if not self.config.has_section("metron"):
            self.config.add_section("metron")

        self.config["DEFAULT"]["sort_dir"] = self.sort_dir
        self.config["metron"]["user"] = self.metron_user
        self.config["metron"]["password"] = self.metron_pass

        with open(self.settings_file, "w") as configfile:
            self.config.write(configfile)



import os
import tempfile
from shutil import make_archive
from unittest import TestCase, main

from metrontagger.comicapi.comicarchive import ComicArchive, MetaDataStyle
from metrontagger.comicapi.genericmetadata import GenericMetadata
from metrontagger.taggerlib.filesorter import FileSorter


class TestFileSorter(TestCase):
    def setUp(self):
        self.tmp_archive_dir = tempfile.TemporaryDirectory()
        self.tmp_image_dir = tempfile.TemporaryDirectory()
        self.tmp_sort_dir = tempfile.TemporaryDirectory()
        # Create 3 fake jpgs
        img_1 = tempfile.NamedTemporaryFile(
            suffix=".jpg", dir=self.tmp_image_dir.name, mode="wb"
        )
        img_1.write(b"test data")
        img_2 = tempfile.NamedTemporaryFile(
            suffix=".jpg", dir=self.tmp_image_dir.name, mode="wb"
        )
        img_2.write(b"more data")
        img_3 = tempfile.NamedTemporaryFile(
            suffix=".jpg", dir=self.tmp_image_dir.name, mode="wb"
        )
        img_3.write(b"yet more data")

        self.zfile = os.path.join(self.tmp_archive_dir.name, "comic")

        # Create zipfile
        open(make_archive(self.zfile, "zip", self.tmp_image_dir.name), "rb").read()

        # Create test metadata
        md = GenericMetadata()
        md.series = "Aquaman"
        md.issue = "1"
        md.year = "2011"
        md.publisher = "DC Comics"

        # Now write it to the zipfile
        ca = ComicArchive(self.zfile + ".zip")
        ca.writeMetadata(md, MetaDataStyle.CIX)

    def tearDown(self):
        self.tmp_archive_dir.cleanup()
        self.tmp_sort_dir.cleanup()
        self.tmp_image_dir.cleanup()

    def test_sort_file(self):
        fs = FileSorter(self.tmp_sort_dir.name)
        res = fs.sort_comics(self.zfile + ".zip")
        self.assertTrue(res)


if __name__ == "__main__":
    main()



from unittest import TestCase, main

from metrontagger.comicapi.issuestring import IssueString


class TestIssueString(TestCase):
    def test_issue_string_pad(self):
        val = IssueString(int(1)).asString(pad=3)
        self.assertEqual(val, "001")

    def test_issue_float(self):
        val = IssueString("1½").asFloat()
        self.assertEqual(val, 1.5)

    def test_issue_float_half(self):
        val = IssueString("½").asFloat()
        self.assertEqual(val, 0.5)

    def test_issue_verify_float(self):
        val = IssueString("1.5").asFloat()
        self.assertEqual(val, 1.5)

    def test_issue_string_no_value_as_int(self):
        val = IssueString("").asInt()
        self.assertIsNone(val)

    def test_issue_int(self):
        val = IssueString("1").asInt()
        self.assertEqual(val, 1)

    def test_issue_float_as_int(self):
        val = IssueString("1.5").asInt()
        self.assertEqual(val, 1)

    def test_issue_string_monsters_unleashed(self):
        val = IssueString("1.MU").asString(3)
        self.assertEqual(val, "001.MU")

    def test_issue_string_minus_one(self):
        val = IssueString("-1").asString(3)
        self.assertEqual(val, "-001")

    def test_issue_string_none_value(self):
        val = IssueString("Test").asString()
        self.assertEqual(val, "Test")


if __name__ == "__main__":
    main()



import os
import tempfile
import unittest

from metrontagger.comicapi import utils


class TestUtils(unittest.TestCase):
    def test_remove_articles(self):
        txt = "The Champions & Inhumans"
        new_txt = utils.removearticles(txt)
        self.assertEqual(new_txt, "champions inhumans")

    def test_list_to_string(self):
        thislist = ["apple", "banana", "cherry"]
        expected_result = "apple; banana; cherry"

        list_string = utils.listToString(thislist)
        self.assertEqual(list_string, expected_result)

    def test_unique_name(self):
        tmp_file = tempfile.NamedTemporaryFile(suffix=".cbz")
        new_file = tmp_file.name
        new_name = utils.unique_file(new_file)
        # Now let's create our expected result
        result_split = os.path.splitext(tmp_file.name)
        correct_result = result_split[0] + " (1)" + result_split[1]
        # Don't forget to close the tmpfile
        tmp_file.close()
        self.assertEqual(new_name, correct_result)


if __name__ == "__main__":
    unittest.main()



from base64 import standard_b64encode
from unittest import TestCase, main
from unittest.mock import patch

from metrontagger.comicapi.genericmetadata import GenericMetadata
from metrontagger.taggerlib.metrontalker import MetronTalker


class TestMetronTalker(TestCase):
    def setUp(self):
        auth = f"test_user:test_auth"
        self.base64string = standard_b64encode(auth.encode("utf-8"))
        self.talker = MetronTalker(self.base64string)
        self.resp = {
            "id": 1778,
            "publisher": {"id": 2, "name": "DC Comics"},
            "series": {"id": 204, "name": "Aquaman"},
            "volume": 4,
            "number": "0",
            "name": ["A Crash of Symbols"],
            "cover_date": "1994-10-01",
            "store_date": None,
            "desc": "A ZERO HOUR tie-in! Aquaman battles against his father and the outcome leads to his discovery of a hidden heritage. When the water subsides, the Sea King's relationship with the surface world and his life as a hero will be forever affected!",
            "image": "http://127.0.0.1:8000/media/issue/2019/04/01/aquaman-0.jpg",
            "arcs": [{"id": 69, "name": "Zero Hour"}],
            "credits": [
                {
                    "id": 1576,
                    "creator": "Brad Vancata",
                    "role": [{"id": 4, "name": "Inker"}],
                },
                {
                    "id": 1575,
                    "creator": "Dan Nakrosis",
                    "role": [{"id": 6, "name": "Letterer"}],
                },
                {
                    "id": 739,
                    "creator": "Eddie Berganza",
                    "role": [{"id": 12, "name": "Assistant Editor"}],
                },
                {
                    "id": 1579,
                    "creator": "Howard Shum",
                    "role": [{"id": 4, "name": "Inker"}],
                },
                {
                    "id": 1560,
                    "creator": "Kevin Dooley",
                    "role": [{"id": 8, "name": "Editor"}],
                },
                {
                    "id": 1577,
                    "creator": "Martin Egeland",
                    "role": [
                        {"id": 3, "name": "Penciller"},
                        {"id": 7, "name": "Cover"},
                    ],
                },
                {
                    "id": 166,
                    "creator": "Peter David",
                    "role": [{"id": 1, "name": "Writer"}],
                },
                {
                    "id": 1268,
                    "creator": "Tom McCraw",
                    "role": [{"id": 5, "name": "Colorist"}],
                },
            ],
            "characters": [
                {"id": 86, "name": "Aquaman"},
                {"id": 1499, "name": "Dolphin"},
                {"id": 416, "name": "Garth"},
                {"id": 1500, "name": "Vulko"},
            ],
            "teams": [],
        }

    def test_map_resp_to_metadata(self):
        md = self.talker.mapMetronDataToMetadata(self.resp)
        self.assertIsNotNone(md)
        self.assertEqual(md.series, self.resp["series"]["name"])
        self.assertEqual(md.volume, self.resp["volume"])
        self.assertEqual(md.publisher, self.resp["publisher"]["name"])
        self.assertEqual(md.issue, self.resp["number"])
        self.assertEqual(md.year, "1994")

    @patch("metrontagger.taggerlib.metrontalker.MetronTalker.fetchResponse")
    def test_fetch_issue_by_id(self, MockFetch):
        MockFetch.return_value = self.resp
        talker = MetronTalker(self.base64string)
        md = talker.fetchIssueDataByIssueId("1")
        self.assertIsNotNone(md)
        self.assertIsInstance(md, GenericMetadata)
        self.assertEqual(md.series, self.resp["series"]["name"])
        self.assertEqual(md.issue, self.resp["number"])

    @patch("metrontagger.taggerlib.metrontalker.MetronTalker.fetchResponse")
    def test_search_for_issue(self, MockFetch):
        query_dict = {"series": "aquaman", "volume": "", "number": "10", "year": ""}
        res = {
            "count": 7,
            "next": None,
            "previous": None,
            "results": [
                {"id": 3643, "__str__": "Aquaman #10", "cover_date": "1963-08-01"},
                {"id": 2519, "__str__": "Aquaman #10", "cover_date": "1992-09-01"},
                {"id": 1786, "__str__": "Aquaman #10", "cover_date": "1995-07-01"},
                {"id": 2439, "__str__": "Aquaman #10", "cover_date": "2003-11-01"},
                {"id": 2534, "__str__": "Aquaman #10", "cover_date": "2012-08-01"},
                {"id": 2908, "__str__": "Aquaman #10", "cover_date": "2017-01-01"},
                {
                    "id": 3631,
                    "__str__": "Aquaman and the Others #10",
                    "cover_date": "2014-04-01",
                },
            ],
        }
        MockFetch.return_value = res
        talker = MetronTalker(self.base64string)
        response = talker.searchForIssue(query_dict)
        self.assertIsNotNone(response)
        self.assertEqual(response, res)


if __name__ == "__main__":
    main()



import os
import sys
from base64 import standard_b64encode

# Append sys.path so imports work.
sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

from metrontagger.comicapi.comicarchive import ComicArchive, MetaDataStyle
from metrontagger.comicapi.genericmetadata import GenericMetadata
from metrontagger.comicapi.utils import get_recursive_filelist, unique_file
from metrontagger.taggerlib.filerenamer import FileRenamer
from metrontagger.taggerlib.filesorter import FileSorter
from metrontagger.taggerlib.metrontalker import MetronTalker
from metrontagger.taggerlib.options import make_parser
from metrontagger.taggerlib.settings import MetronTaggerSettings
from metrontagger.taggerlib.utils import create_issue_query_dict


# Load the settings
SETTINGS = MetronTaggerSettings()


class MultipleMatch:
    def __init__(self, filename, match_list):
        self.filename = filename
        self.matches = match_list


class OnlineMatchResults:
    def __init__(self):
        self.goodMatches = []
        self.noMatches = []
        self.multipleMatches = []


def create_metron_talker():
    auth = f"{SETTINGS.metron_user}:{SETTINGS.metron_pass}"
    base64string = standard_b64encode(auth.encode("utf-8"))
    talker = MetronTalker(base64string)

    return talker


def createPagelistMetadata(ca):
    md = GenericMetadata()
    md.setDefaultPageList(ca.getNumberOfPages())

    return md


def getIssueMetadata(filename, issue_id, talker):
    success = False

    metron_md = talker.fetchIssueDataByIssueId(issue_id)
    if metron_md:
        ca = ComicArchive(filename)
        md = createPagelistMetadata(ca)
        md.overlay(metron_md)
        ca.writeMetadata(md, MetaDataStyle.CIX)
        success = True

    return success


def selectChoiceFromMultipleMatches(filename, match_set):
    print(f"{os.path.basename(filename)} - Multiple results found")

    # sort match list by cover date
    match_set = sorted(match_set, key=lambda m: m["cover_date"])

    for (counter, m) in enumerate(match_set, start=1):
        print(f"{counter}. {m['__str__']} ({m['cover_date']})")

    while True:
        i = input("Choose a match #, or 's' to skip: ")
        if (i.isdigit() and int(i) in range(1, len(match_set) + 1)) or i == "s":
            break

    if i != "s":
        i = int(i) - 1
        issue_id = match_set[i]["id"]
    else:
        issue_id = None

    return issue_id


def processFile(filename, match_results, talker, ignore):
    ca = ComicArchive(filename)

    if not ca.seemsToBeAComicArchive():
        print(f"{os.path.basename(filename)} does not appear to be a comic archive.")
        return None, False

    if ignore:
        if ca.hasCIX():
            print(f"{os.path.basename(filename)} has metadata. Skipping...")
            return None, False

    if not ca.isWritable():
        print(f"{os.path.basename(filename)} is not writable.")
        return None, False

    query_dict = create_issue_query_dict(filename)
    res = talker.searchForIssue(query_dict)
    res_count = res["count"]

    issue_id = None
    multiple_match = False
    if not res_count > 0:
        issue_id = None
        match_results.noMatches.append(filename)
        multiple_match = False
    elif res_count > 1:
        issue_id = None
        match_results.multipleMatches.append(MultipleMatch(filename, res["results"]))
        multiple_match = True
    elif res_count == 1:
        issue_id = res["results"][0]["id"]
        match_results.goodMatches.append(filename)
        multiple_match = False

    return issue_id, multiple_match


def postProcessMatches(match_results, talker):
    # Print file matching results.
    if len(match_results.goodMatches) > 0:
        print("\nSuccessful matches:\n------------------")
        for f in match_results.goodMatches:
            print(f)

    if len(match_results.noMatches) > 0:
        print("\nNo matches:\n------------------")
        for f in match_results.noMatches:
            print(f)

    # Handle files with multiple matches.
    if len(match_results.multipleMatches) > 0:
        for match_set in match_results.multipleMatches:
            issue_id = selectChoiceFromMultipleMatches(
                match_set.filename, match_set.matches
            )
            if issue_id:
                success = getIssueMetadata(match_set.filename, issue_id, talker)
                if not success:
                    print(
                        f"Unable to retrieve metadata for '{os.path.basename(match_set.filename)}'."
                    )


def main():

    parser = make_parser()
    opts = parser.parse_args()

    if opts.user:
        SETTINGS.metron_user = opts.user

    if opts.password:
        SETTINGS.metron_pass = opts.password

    if opts.sort_dir:
        SETTINGS.sort_dir = opts.sort_dir

    if opts.set_metron_user or opts.set_sort_dir:
        SETTINGS.save()

    # Parse paths to get file list
    file_list = []
    file_list = get_recursive_filelist(opts.path)

    if not file_list:
        print("No files to process. Exiting.")
        sys.exit(0)

    if opts.missing:
        print("** Showing files without metadata **")
        for f in file_list:
            ca = ComicArchive(f)
            if ca.hasMetadata(MetaDataStyle.CIX):
                continue
            print(f"no metadata in '{os.path.basename(f)}'")

    if opts.delete:
        print("** Removing metadata **")
        for f in file_list:
            ca = ComicArchive(f)
            if ca.hasMetadata(MetaDataStyle.CIX):
                ca.removeMetadata(MetaDataStyle.CIX)
                print(f"removed metadata from '{os.path.basename(f)}'.")
            else:
                print(f"no metadata in '{os.path.basename(f)}'.")

    if opts.id:
        if len(file_list) > 1:
            print("More than one file was passed for Id processing. Exiting...")
            sys.exit(0)

        filename = file_list[0]
        talker = create_metron_talker()
        success = getIssueMetadata(filename, opts.id, talker)
        if success:
            print(f"match found for '{os.path.basename(filename)}'.")

    if opts.online:
        print("** Starting online search and tagging **")

        # Initialize class to handle results for files with multiple matches
        match_results = OnlineMatchResults()
        talker = create_metron_talker()

        # Let's look online to see if we can find any matches on Metron.
        for filename in file_list:
            issue_id, multiple_match = processFile(
                filename, match_results, talker, opts.ignore_existing
            )
            if issue_id:
                success = getIssueMetadata(filename, issue_id, talker)
                if success:
                    print(f"match found for '{os.path.basename(filename)}'.")
                else:
                    print(
                        f"there was a problem writing metadate for '{os.path.basename(filename)}'."
                    )
            else:
                if not multiple_match:
                    print(f"no match for '{os.path.basename(filename)}'.")
                    continue

        # Print match results & handle files with multiple matches
        postProcessMatches(match_results, talker)

    if opts.rename:
        print("** Starting comic archive renaming **")
        for f in file_list:
            ca = ComicArchive(f)
            if not ca.hasCIX():
                print(f"skipping '{os.path.basename(f)}'. No metadata available.")
                continue

            md = ca.readMetadata(MetaDataStyle.CIX)
            renamer = FileRenamer(md)
            new_name = renamer.determineName(f)

            if new_name == os.path.basename(f):
                print("Filename is already good!", file=sys.stderr)
                continue

            folder = os.path.dirname(os.path.abspath(f))
            new_abs_path = unique_file(os.path.join(folder, new_name))
            os.rename(f, new_abs_path)
            print(f"renamed '{os.path.basename(f)}' -> '{new_name}'")

    if opts.sort:
        if not SETTINGS.sort_dir:
            print("Unable to sort files. No destination directory was provided.")
            return

        print("** Starting sorting of comic archives **")
        fs = FileSorter(SETTINGS.sort_dir)
        for comic in file_list:
            result = fs.sort_comics(comic)
            if not result:
                print(f"unable to move {os.path.basename(comic)}.")


if __name__ == "__main__":
    sys.exit(main())



import os
from shutil import Error, move

from metrontagger.comicapi.comicarchive import ComicArchive, MetaDataStyle
from metrontagger.taggerlib.utils import cleanup_string


class FileSorter:
    def __init__(self, directory):
        self.set_directory(directory)

    def set_directory(self, directory):
        self.sort_directory = directory

    def set_template(self, template):
        self.template = template

    def sort_comics(self, comic):
        ca = ComicArchive(comic)
        if ca.hasMetadata(MetaDataStyle.CIX):
            md = ca.readMetadata(MetaDataStyle.CIX)
        else:
            return False

        if md is not None:
            # Cleanup the publisher & series metadata so they play nicely with filesystems.
            publisher = cleanup_string(md.publisher)
            series = cleanup_string(md.series)
            if md.volume:
                volume = "v" + cleanup_string(md.volume)
                new_path = (
                    self.sort_directory
                    + os.sep
                    + publisher
                    + os.sep
                    + series
                    + os.sep
                    + volume
                    + os.sep
                )
            else:
                new_path = (
                    self.sort_directory + os.sep + publisher + os.sep + series + os.sep
                )
        else:
            return False

        if not os.path.isdir(new_path):
            os.makedirs(new_path)

        try:
            move(comic, new_path)
            print(f"moved {os.path.basename(comic)} to {new_path}.")
        except Error:
            return False

        return True



import urllib.parse

from metrontagger.comicapi.filenameparser import FileNameParser


def cleanup_string(path_name):
    path_name = path_name.replace("/", "-")
    path_name = path_name.replace(" :", " -")
    path_name = path_name.replace(": ", " - ")
    path_name = path_name.replace(":", "-")
    path_name = path_name.replace("?", "")

    return path_name


def create_issue_query_dict(filename):
    fnp = FileNameParser()
    fnp.parseFilename(filename)

    # Substitute colon for hyphen when searching for series name
    fixed_txt = fnp.series.replace(" - ", ": ")
    series_word_list = fixed_txt.split()
    series_string = " ".join(series_word_list).strip()
    series_string = urllib.parse.quote_plus(series_string.encode("utf-8"))
    number = urllib.parse.quote_plus(fnp.issue.encode("utf-8"))
    query_dict = {
        "series": series_string,
        "volume": fnp.volume,
        "number": number,
        "year": fnp.year,
    }

    return query_dict



import tempfile
from unittest import TestCase, main

from metrontagger.comicapi.comicinfoxml import ComicInfoXml
from metrontagger.comicapi.genericmetadata import GenericMetadata


class TestComicInfoXml(TestCase):
    def setUp(self):
        self.md = GenericMetadata()
        self.md.series = "Aquaman"
        self.md.issue = "1"
        self.md.year = "1993"
        self.md.day = "15"
        self.md.addCredit("Peter David", "Writer", primary=True)
        self.md.addCredit("Martin Egeland", "Penciller")
        self.md.addCredit("Martin Egeland", "Cover")
        self.md.addCredit("Kevin Dooley", "Editor")
        self.md.addCredit("Howard Shum", "Inker")
        self.md.addCredit("Tom McCraw", "Colorist")
        self.md.addCredit("Dan Nakrosis", "Letterer")

    def test_metadata_from_xml(self):
        res = ComicInfoXml().stringFromMetadata(self.md)
        # TODO: add more asserts to verify data.
        self.assertIsNotNone(res)

    def test_meta_write_to_file(self):
        tmp_file = tempfile.NamedTemporaryFile(suffix=".xml")
        ComicInfoXml().writeToExternalFile(tmp_file.name, self.md)
        # Read the contents of the file just written.
        # TODO: Verify the data.
        res = open(tmp_file.name).read()
        tmp_file.close()
        self.assertIsNotNone(res)

    def test_read_from_file(self):
        tmp_file = tempfile.NamedTemporaryFile(suffix=".xml")
        # Write metadata to file
        ComicInfoXml().writeToExternalFile(tmp_file.name, self.md)
        # Read the metadat from the file
        new_md = ComicInfoXml().readFromExternalFile(tmp_file.name)
        tmp_file.close()

        self.assertIsNotNone(new_md)
        self.assertEqual(new_md.series, self.md.series)
        self.assertEqual(new_md.issue, self.md.issue)
        self.assertEqual(new_md.year, self.md.year)
        self.assertEqual(new_md.month, self.md.month)
        self.assertEqual(new_md.day, self.md.day)


if __name__ == "__main__":
    main()



import tempfile
from unittest import TestCase, main

from metrontagger.taggerlib.settings import MetronTaggerSettings


class TestSettings(TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.TemporaryDirectory()

    def tearDown(self):
        self.tmp_dir.cleanup()

    def test_settings(self):
        user = "test"
        dummy = "dummy_value"
        # Save a test config file
        config = MetronTaggerSettings(config_dir=self.tmp_dir.name)
        config.metron_user = user
        config.metron_pass = dummy
        config.save()

        # Now load that file and verify the contents
        new_config = MetronTaggerSettings(config_dir=self.tmp_dir.name)
        self.assertEqual(new_config.metron_user, user)
        self.assertEqual(new_config.metron_pass, dummy)


if __name__ == "__main__":
    main()



"""Functions for renaming files based on metadata"""

# Copyright 2012-2014 Anthony Beville

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import os
import re

from metrontagger.comicapi.issuestring import IssueString


class FileRenamer:
    def __init__(self, metadata):
        self.setMetadata(metadata)
        self.setTemplate("%series% v%volume% #%issue% (of %issuecount%) (%year%)")
        self.smart_cleanup = True
        self.issue_zero_padding = 3

    def setMetadata(self, metadata):
        self.metdata = metadata

    def setIssueZeroPadding(self, count):
        self.issue_zero_padding = count

    def setSmartCleanup(self, on):
        self.smart_cleanup = on

    def setTemplate(self, template):
        self.template = template

    def replaceToken(self, text, value, token):
        # helper func
        def isToken(word):
            return word[0] == "%" and word[-1:] == "%"

        if value is not None:
            return text.replace(token, str(value))
        else:
            if self.smart_cleanup:
                # smart cleanup means we want to remove anything appended to token if it's empty
                # (e.g "#%issue%"  or "v%volume%")
                # (TODO: This could fail if there is more than one token appended together, I guess)
                text_list = text.split()

                # special case for issuecount, remove preceding non-token word,
                # as in "...(of %issuecount%)..."
                if token == "%issuecount%":
                    for idx, word in enumerate(text_list):
                        if token in word and not isToken(text_list[idx - 1]):
                            text_list[idx - 1] = ""

                text_list = [x for x in text_list if token not in x]
                return " ".join(text_list)
            else:
                return text.replace(token, "")

    def determineName(self, filename, ext=None):

        md = self.metdata
        new_name = self.template

        new_name = self.replaceToken(new_name, md.series, "%series%")
        new_name = self.replaceToken(new_name, md.volume, "%volume%")

        if md.issue is not None:
            issue_str = "{0}".format(
                IssueString(md.issue).asString(pad=self.issue_zero_padding)
            )
        else:
            issue_str = None
        new_name = self.replaceToken(new_name, issue_str, "%issue%")

        new_name = self.replaceToken(new_name, md.issueCount, "%issuecount%")
        new_name = self.replaceToken(new_name, md.year, "%year%")
        new_name = self.replaceToken(new_name, md.publisher, "%publisher%")
        new_name = self.replaceToken(new_name, md.title, "%title%")
        new_name = self.replaceToken(new_name, md.month, "%month%")
        month_name = None
        if md.month is not None:
            if (isinstance(md.month, str) and md.month.isdigit()) or isinstance(
                md.month, int
            ):
                if int(md.month) in range(1, 13):
                    dt = datetime.datetime(1970, int(md.month), 1, 0, 0)
                    month_name = dt.strftime("%B")
        new_name = self.replaceToken(new_name, month_name, "%month_name%")

        new_name = self.replaceToken(new_name, md.genre, "%genre%")
        new_name = self.replaceToken(new_name, md.language, "%language_code%")
        new_name = self.replaceToken(new_name, md.criticalRating, "%criticalrating%")
        new_name = self.replaceToken(new_name, md.alternateSeries, "%alternateseries%")
        new_name = self.replaceToken(new_name, md.alternateNumber, "%alternatenumber%")
        new_name = self.replaceToken(new_name, md.alternateCount, "%alternatecount%")
        new_name = self.replaceToken(new_name, md.imprint, "%imprint%")
        new_name = self.replaceToken(new_name, md.format, "%format%")
        new_name = self.replaceToken(new_name, md.maturityRating, "%maturityrating%")
        new_name = self.replaceToken(new_name, md.storyArc, "%storyarc%")
        new_name = self.replaceToken(new_name, md.seriesGroup, "%seriesgroup%")
        new_name = self.replaceToken(new_name, md.scanInfo, "%scaninfo%")

        if self.smart_cleanup:

            # remove empty braces,brackets, parentheses
            new_name = re.sub("\(\s*[-:]*\s*\)", "", new_name)
            new_name = re.sub("\[\s*[-:]*\s*\]", "", new_name)
            new_name = re.sub("\{\s*[-:]*\s*\}", "", new_name)

            # remove duplicate spaces
            new_name = " ".join(new_name.split())

            # remove remove duplicate -, _,
            new_name = re.sub("[-_]{2,}\s+", "-- ", new_name)
            new_name = re.sub("(\s--)+", " --", new_name)
            new_name = re.sub("(\s-)+", " -", new_name)

            # remove dash or double dash at end of line
            new_name = re.sub("[-]{1,2}\s*$", "", new_name)

            # remove duplicate spaces (again!)
            new_name = " ".join(new_name.split())

        if ext is None:
            ext = os.path.splitext(filename)[1]

        new_name += ext

        # some tweaks to keep various filesystems happy
        new_name = new_name.replace("/", "-")
        new_name = new_name.replace(" :", " -")
        new_name = new_name.replace(": ", " - ")
        new_name = new_name.replace(":", "-")
        new_name = new_name.replace("?", "")

        return new_name



from unittest import TestCase, main

from metrontagger.comicapi.genericmetadata import GenericMetadata


class TestGenericMetadata(TestCase):
    def setUp(self):
        self.md = GenericMetadata()
        self.md.series = "Aquaman"
        self.md.issue = "0"
        self.md.title = "A Crash of Symbols"
        self.md.isEmpty = False

        self.new_md = GenericMetadata()
        self.new_md.year = "1994"
        self.new_md.month = "10"
        self.new_md.day = "1"
        self.md.isEmpty = False

    def test_metadata_overlay(self):
        self.md.overlay(self.new_md)

        self.assertEqual(self.md.series, "Aquaman")
        self.assertEqual(self.md.issue, "0")
        self.assertEqual(self.md.title, "A Crash of Symbols")
        self.assertEqual(self.md.year, "1994")
        self.assertEqual(self.md.month, "10")
        self.assertEqual(self.md.day, "1")

    def test_metadata_credits(self):
        result = [
            {"person": "Peter David", "primary": True, "role": "Writer"},
            {"person": "Martin Egeland", "role": "Penciller"},
            {"person": "Martin Egeland", "role": "Cover"},
        ]

        self.md.addCredit("Peter David", "Writer", primary=True)
        self.md.addCredit("Martin Egeland", "Penciller")
        self.md.addCredit("Martin Egeland", "Cover")

        self.assertEqual(self.md.credits, result)

    def test_metadata_credits_overlay(self):
        new_credit = [{"person": "Tom McCray", "role": "Colorist"}]
        result = [
            {"person": "Peter David", "role": "Writer"},
            {"person": "Tom McCray", "role": "Colorist"},
        ]

        self.md.addCredit("Peter David", "Writer")
        self.md.overlayCredits(new_credit)

        self.assertEqual(self.md.credits, result)

    def test_metadata_print_str(self):
        self.assertEqual(
            str(self.md), "series: Aquaman\nissue:  0\ntitle:  A Crash of Symbols\n"
        )

    def test_no_metadata_print_str(self):
        md = GenericMetadata()
        self.assertEqual(str(md), "No metadata")


if __name__ == "__main__":
    main()



"""A class to represent a single comic, be it file or folder of images"""

# Copyright 2012-2014 Anthony Beville

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import io
import os
import sys
import tempfile
import zipfile

from natsort import natsorted

from .comicinfoxml import ComicInfoXml
from .filenameparser import FileNameParser
from .genericmetadata import GenericMetadata

try:
    from PIL import Image

    pil_available = True
except ImportError:
    pil_available = False


sys.path.insert(0, os.path.abspath("."))


class MetaDataStyle:
    # Only have ComicRack is supported for now, but if we want to add back other formats this will be useful.
    CIX = 0
    name = ["ComicRack"]


class ZipArchiver:

    """ZIP implementation"""

    def __init__(self, path):
        self.path = path

    def readArchiveFile(self, archive_file):
        data = ""
        zf = zipfile.ZipFile(self.path, "r")

        try:
            data = zf.read(archive_file)
        except zipfile.BadZipfile as e:
            print(f"bad zipfile [{e}]: {self.path} :: {archive_file}", file=sys.stderr)
            zf.close()
            raise IOError
        except Exception as e:
            zf.close()
            print(f"bad zipfile [{e}]: {self.path} :: {archive_file}", file=sys.stderr)
            raise IOError
        finally:
            zf.close()
        return data

    def removeArchiveFile(self, archive_file):
        try:
            self.rebuildZipFile([archive_file])
        except zipfile.BadZipfile:
            return False
        else:
            return True

    def writeArchiveFile(self, archive_file, data):
        #  At the moment, no other option but to rebuild the whole
        #  zip archive w/o the indicated file. Very sucky, but maybe
        # another solution can be found
        try:
            self.rebuildZipFile([archive_file])

            # now just add the archive file as a new one
            zf = zipfile.ZipFile(
                self.path, mode="a", allowZip64=True, compression=zipfile.ZIP_DEFLATED
            )
            zf.writestr(archive_file, data)
            zf.close()
            return True
        except (zipfile.BadZipfile, zipfile.LargeZipFile) as e:
            print(f"Error writing zipfile: {e}.")
            return False

    def getArchiveFilenameList(self):
        try:
            zf = zipfile.ZipFile(self.path, "r")
            namelist = zf.namelist()
            zf.close()
            return namelist
        except Exception as e:
            print(f"Unable to get zipfile list [{e}]: {self.path}", file=sys.stderr)
            return []

    def rebuildZipFile(self, exclude_list):
        """Zip helper func

        This recompresses the zip archive, without the files in the exclude_list
        """

        # print ">> sys.stderr, Rebuilding zip {0} without {1}".format(
        #                                            self.path, exclude_list )

        # generate temp file
        tmp_fd, tmp_name = tempfile.mkstemp(dir=os.path.dirname(self.path))
        os.close(tmp_fd)

        zin = zipfile.ZipFile(self.path, "r")
        zout = zipfile.ZipFile(tmp_name, "w", allowZip64=True)
        for item in zin.infolist():
            buffer = zin.read(item.filename)
            if item.filename not in exclude_list:
                zout.writestr(item, buffer)

        zout.close()
        zin.close()

        # replace with the new file
        os.remove(self.path)
        os.rename(tmp_name, self.path)

    def copyFromArchive(self, otherArchive):
        """Replace the current zip with one copied from another archive"""

        try:
            zout = zipfile.ZipFile(self.path, "w", allowZip64=True)
            for fname in otherArchive.getArchiveFilenameList():
                data = otherArchive.readArchiveFile(fname)
                if data is not None:
                    zout.writestr(fname, data)
            zout.close()
        except Exception as e:
            print(f"Error while copying to {self.path}: {e}", file=sys.stderr)
            return False
        else:
            return True


# ------------------------------------------


class UnknownArchiver:

    """Unknown implementation"""

    def __init__(self, path):
        self.path = path

    @classmethod
    def readArchiveFile(self):
        return ""

    @classmethod
    def writeArchiveFile(self, archive_file, data):
        return False

    @classmethod
    def removeArchiveFile(self, archive_file):
        return False

    @classmethod
    def getArchiveFilenameList(self):
        return []


# ------------------------------------------------------------------


class ComicArchive:
    class ArchiveType:
        Zip, Unknown = list(range(2))

    def __init__(self, path):
        self.path = path

        self.ci_xml_filename = "ComicInfo.xml"
        self.resetCache()

        self.archive_type = self.ArchiveType.Unknown
        self.archiver = UnknownArchiver(self.path)

        if self.zipTest():
            self.archive_type = self.ArchiveType.Zip
            self.archiver = ZipArchiver(self.path)

    def resetCache(self):
        """Clears the cached data"""
        self.has_cix = None
        self.page_count = None
        self.page_list = None
        self.cix_md = None

    def loadCache(self, style_list):
        for style in style_list:
            self.readMetadata(style)

    def rename(self, path):
        self.path = path
        self.archiver.path = path

    def zipTest(self):
        return zipfile.is_zipfile(self.path)

    def isZip(self):
        return self.archive_type == self.ArchiveType.Zip

    def isWritable(self):
        if self.archive_type == self.ArchiveType.Unknown:
            return False

        elif not os.access(self.path, os.W_OK):
            return False

        return True

    def isWritableForStyle(self, data_style):

        return self.isWritable()

    def seemsToBeAComicArchive(self):

        if (self.isZip()) and (self.getNumberOfPages() > 0):
            return True
        else:
            return False

    def readMetadata(self, style):

        if style == MetaDataStyle.CIX:
            return self.readCIX()
        else:
            return GenericMetadata()

    def writeMetadata(self, metadata, style):

        retcode = None
        if style == MetaDataStyle.CIX:
            retcode = self.writeCIX(metadata)
        return retcode

    def hasMetadata(self, style):

        if style == MetaDataStyle.CIX:
            return self.hasCIX()
        else:
            return False

    def removeMetadata(self, style):
        retcode = True
        if style == MetaDataStyle.CIX:
            retcode = self.removeCIX()
        return retcode

    def getPage(self, index):

        image_data = None

        filename = self.getPageName(index)

        if filename is not None:
            try:
                image_data = self.archiver.readArchiveFile(filename)
            except IOError:
                print("Error reading in page.", file=sys.stderr)

        return image_data

    def getPageName(self, index):

        if index is None:
            return None

        page_list = self.getPageNameList()

        num_pages = len(page_list)
        if num_pages == 0 or index >= num_pages:
            return None

        return page_list[index]

    def getScannerPageIndex(self):

        scanner_page_index = None

        # make a guess at the scanner page
        name_list = self.getPageNameList()
        count = self.getNumberOfPages()

        # too few pages to really know
        if count < 5:
            return None

        # count the length of every filename, and count occurences
        length_buckets = dict()
        for name in name_list:
            fname = os.path.split(name)[1]
            length = len(fname)
            if length in length_buckets:
                length_buckets[length] += 1
            else:
                length_buckets[length] = 1

        # sort by most common
        sorted_buckets = sorted(
            iter(length_buckets.items()), key=lambda k_v: (k_v[1], k_v[0]), reverse=True
        )

        # statistical mode occurence is first
        mode_length = sorted_buckets[0][0]

        # we are only going to consider the final image file:
        final_name = os.path.split(name_list[count - 1])[1]

        common_length_list = list()
        for name in name_list:
            if len(os.path.split(name)[1]) == mode_length:
                common_length_list.append(os.path.split(name)[1])

        prefix = os.path.commonprefix(common_length_list)

        if mode_length <= 7 and prefix == "":
            # probably all numbers
            if len(final_name) > mode_length:
                scanner_page_index = count - 1

        # see if the last page doesn't start with the same prefix as most
        # others
        elif not final_name.startswith(prefix):
            scanner_page_index = count - 1

        return scanner_page_index

    def getPageNameList(self, sort_list=True):

        if self.page_list is None:
            # get the list file names in the archive, and sort
            files = self.archiver.getArchiveFilenameList()

            # seems like some archive creators are on  Windows, and don't know
            # about case-sensitivity!
            if sort_list:

                def keyfunc(k):
                    # hack to account for some weird scanner ID pages
                    # basename=os.path.split(k)[1]
                    # if basename < '0':
                    #    k = os.path.join(os.path.split(k)[0], "z" + basename)
                    return k.lower()

                files = natsorted(files, key=keyfunc)

            # make a sub-list of image files
            self.page_list = []
            for name in files:
                if (
                    name[-4:].lower() in [".jpg", "jpeg", ".png", ".gif", "webp"]
                    and os.path.basename(name)[0] != "."
                ):
                    self.page_list.append(name)

        return self.page_list

    def getNumberOfPages(self):

        if self.page_count is None:
            self.page_count = len(self.getPageNameList())
        return self.page_count

    def readCIX(self):
        if self.cix_md is None:
            raw_cix = self.readRawCIX()
            if raw_cix is None or raw_cix == "":
                self.cix_md = GenericMetadata()
            else:
                self.cix_md = ComicInfoXml().metadataFromString(raw_cix)

            # validate the existing page list (make sure count is correct)
            if len(self.cix_md.pages) != 0:
                if len(self.cix_md.pages) != self.getNumberOfPages():
                    # pages array doesn't match the actual number of images we're seeing
                    # in the archive, so discard the data
                    self.cix_md.pages = []

            if len(self.cix_md.pages) == 0:
                self.cix_md.setDefaultPageList(self.getNumberOfPages())

        return self.cix_md

    def readRawCIX(self):
        if not self.hasCIX():
            return None
        try:
            raw_cix = self.archiver.readArchiveFile(self.ci_xml_filename)
        except IOError:
            print("Error reading in raw CIX!")
            raw_cix = ""
        return raw_cix

    def writeCIX(self, metadata):

        if metadata is not None:
            self.applyArchiveInfoToMetadata(metadata, calc_page_sizes=True)
            cix_string = ComicInfoXml().stringFromMetadata(metadata)
            write_success = self.archiver.writeArchiveFile(
                self.ci_xml_filename, cix_string
            )
            if write_success:
                self.has_cix = True
                self.cix_md = metadata
            self.resetCache()
            return write_success
        else:
            return False

    def removeCIX(self):
        if self.hasCIX():
            write_success = self.archiver.removeArchiveFile(self.ci_xml_filename)
            if write_success:
                self.has_cix = False
                self.cix_md = None
            self.resetCache()
            return write_success
        return True

    def hasCIX(self):
        if self.has_cix is None:

            if not self.seemsToBeAComicArchive():
                self.has_cix = False
            elif self.ci_xml_filename in self.archiver.getArchiveFilenameList():
                self.has_cix = True
            else:
                self.has_cix = False
        return self.has_cix

    def applyArchiveInfoToMetadata(self, md, calc_page_sizes=False):
        md.pageCount = self.getNumberOfPages()

        if calc_page_sizes:
            for p in md.pages:
                idx = int(p["Image"])
                if pil_available:
                    if (
                        "ImageSize" not in p
                        or "ImageHeight" not in p
                        or "ImageWidth" not in p
                    ):
                        data = self.getPage(idx)
                        if data is not None:
                            try:
                                im = Image.open(io.BytesIO(data))
                                w, h = im.size

                                p["ImageSize"] = str(len(data))
                                p["ImageHeight"] = str(h)
                                p["ImageWidth"] = str(w)
                            except IOError:
                                p["ImageSize"] = str(len(data))

                else:
                    if "ImageSize" not in p:
                        data = self.getPage(idx)
                        p["ImageSize"] = str(len(data))

    def metadataFromFilename(self, parse_scan_info=True):

        metadata = GenericMetadata()

        fnp = FileNameParser()
        fnp.parseFilename(self.path)

        if fnp.issue != "":
            metadata.issue = fnp.issue
        if fnp.series != "":
            metadata.series = fnp.series
        if fnp.volume != "":
            metadata.volume = fnp.volume
        if fnp.year != "":
            metadata.year = fnp.year
        if fnp.issue_count != "":
            metadata.issueCount = fnp.issue_count
        if parse_scan_info:
            if fnp.remainder != "":
                metadata.scanInfo = fnp.remainder

        metadata.isEmpty = False

        return metadata

    def exportAsZip(self, zipfilename):
        if self.archive_type == self.ArchiveType.Zip:
            # nothing to do, we're already a zip
            return True

        zip_archiver = ZipArchiver(zipfilename)
        return zip_archiver.copyFromArchive(self.archiver)



import os
import tempfile
from unittest import TestCase, main

from metrontagger.comicapi.filenameparser import FileNameParser as fn


class TestFileNameParser(TestCase):
    def setUp(self):
        self.comic = "Afterlife_With_Archie_V1_#002_(of_08)_(2013)"
        self.fnp = fn()

    def test_special_format(self):
        comic = "Aquaman TPB (1994)"
        _, issue_start, _ = self.fnp.getIssueNumber(comic)
        series, volume = self.fnp.getSeriesName(comic, issue_start)
        self.assertEqual(issue_start, 0)
        self.assertEqual(series, "Aquaman")
        self.assertEqual(volume, "1994")

    def test_get_issue_number(self):
        """Returns a tuple of issue number string, and start and end indexes in the filename"""
        issue, issue_start, issue_end = self.fnp.getIssueNumber(self.comic)
        self.assertEqual(issue, "002")
        self.assertEqual(issue_start, 25)
        self.assertEqual(issue_end, 29)

    def test_get_year(self):
        _, _, issue_end = self.fnp.getIssueNumber(self.comic)
        year = self.fnp.getYear(self.comic, issue_end)
        self.assertEqual(year, "2013")

    def test_get_series_name(self):
        _, issue_start, _ = self.fnp.getIssueNumber(self.comic)
        series, volume = self.fnp.getSeriesName(self.comic, issue_start)
        self.assertEqual(series, "Afterlife With Archie")
        self.assertEqual(volume, "1")

    def test_get_count(self):
        _, _, issue_end = self.fnp.getIssueNumber(self.comic)
        issue_count = self.fnp.getIssueCount(self.comic, issue_end)
        self.assertEqual(issue_count, "8")

    def test_fix_spaces(self):
        new_name = self.fnp.fixSpaces(self.comic)
        self.assertNotEqual(new_name, "Afterlife With Archie")

    def test_get_remainder(self):
        _, issue_start, issue_end = self.fnp.getIssueNumber(self.comic)
        year = self.fnp.getYear(self.comic, issue_end)
        _, volume = self.fnp.getSeriesName(self.comic, issue_start)
        count = self.fnp.getIssueCount(self.comic, issue_end)
        remainder = self.fnp.getRemainder(self.comic, year, count, volume, issue_end)
        self.assertEqual(remainder, "(of 08)")

    def test_parse_filename(self):
        tmp_file = tempfile.NamedTemporaryFile(prefix=self.comic, suffix=".cbz")
        tmp_path = tempfile.gettempdir()
        test_filename = tmp_path + os.path.pathsep + tmp_file.name
        tmp_file.close()

        self.fnp.parseFilename(test_filename)
        self.assertEqual(self.fnp.series, "Afterlife With Archie")
        self.assertEqual(self.fnp.volume, "1")
        self.assertEqual(self.fnp.issue, "2")
        self.assertEqual(self.fnp.issue_count, "8")
        self.assertEqual(self.fnp.year, "2013")


if __name__ == "__main__":
    main()



import tempfile
from unittest import TestCase, main

from metrontagger.taggerlib.options import make_parser


class TestOptions(TestCase):
    def setUp(self):
        self.parser = make_parser()
        self.path = tempfile.TemporaryDirectory()

    def tearDown(self):
        self.path.cleanup()

    def test_creditial_options(self):
        parsed = self.parser.parse_args(
            [
                "--user",
                "test_user",
                "--password",
                "test_passwd",
                "--set-metron-user",
                self.path.name,
            ]
        )
        self.assertEqual(parsed.user, "test_user")
        self.assertEqual(parsed.password, "test_passwd")
        self.assertTrue(parsed.set_metron_user)
        self.assertEqual(parsed.path, [self.path.name])

    def test_online_option(self):
        parsed = self.parser.parse_args(["-o", self.path.name])
        self.assertTrue(parsed.online)
        self.assertEqual(parsed.path, [self.path.name])

    def test_id_option(self):
        parsed = self.parser.parse_args(["--id", "1", self.path.name])
        self.assertEqual(parsed.id, "1")
        self.assertEqual(parsed.path, [self.path.name])

    def test_delete_option(self):
        parsed = self.parser.parse_args(["-d", self.path.name])
        self.assertTrue(parsed.delete)
        self.assertEqual(parsed.path, [self.path.name])

    def test_missing_option(self):
        parsed = self.parser.parse_args(["--missing", self.path.name])
        self.assertTrue(parsed.missing)
        self.assertEqual(parsed.path, [self.path.name])

    def test_rename_option(self):
        parsed = self.parser.parse_args(["-r", self.path.name])
        self.assertTrue(parsed.rename)
        self.assertEqual(parsed.path, [self.path.name])

    def test_ignore_option(self):
        parsed = self.parser.parse_args(["--ignore-existing", self.path.name])
        self.assertTrue(parsed.ignore_existing)
        self.assertEqual(parsed.path, [self.path.name])


if __name__ == "__main__":
    main()
